/* ----------------------------------------------------------------------
   The SINUS Firewall -- a TCP/IP packet filter for Linux

   Written within the SINUS project at the University of Zurich,
   SWITCH, Telekurs Payserv AG, ETH Zurich.
   originally based on the sf Firewall Software (C) 1996 by Robert
   Muchsel and Roland Schmid.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   SINUS Firewall resources:
   SINUS Homepage: http://www.ifi.unizh.ch/ikm/SINUS/
   Firewall Homepage: http://www.ifi.unizh.ch/ikm/SINUS/firewall.html
   Frequently asked questions: http://www.ifi.unizh.ch/ikm/SINUS/sf_faq.html
   Mailing list for comments, questions, bug reports: firewall@ifi.unizh.ch
   ----------------------------------------------------------------------  */


/*
 * Routines to send TCP reset for the SF firewall packet filter
 * 
 * $Id: sf_tcp.c,v 1.0 1990/07/25 14:37:44 robby Rel $
 *
 */

#include <linux/config.h>
#ifdef CONFIG_MODVERSIONS
#define MODVERSIONS 1
#endif
#include <linux/modversions.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/errno.h>
#include <linux/net.h>
#include <linux/in.h>
#include <linux/inet.h>
#include <linux/netdevice.h>
#include <linux/icmp.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include <linux/ip.h>
#include <linux/malloc.h>
#include <linux/timer.h>
#include "sf_kernel.h"
#include "sf_rc.h"
#include "sf_global.h"
#include "sf_filter.h"
#include "sf_control.h"
#include "sf_ip.h"
#include "sf_tcp.h"

void send_tcp(int rst, struct device *dev, struct iphdr *ip, struct tcphdr *tcp)
{
  struct sk_buff *skb;
  struct rtable *rt;
  struct tcphdr *tcp2;
  __u32 saddr, daddr;
  register const __u16 *sp;
  register __u32 csum;
  int i, len;

  /* This routine has to be entirely rewritten for Kernel 2.2.x.
   * Currently, it crashes the kernel immediately, and is therefore
   * deactivated.
   */
  return;

  if (dev->flags & IFF_LOOPBACK)	/* Aiiieee :-) */
    return;

  len = sizeof(struct iphdr) + sizeof(struct tcphdr);

  if (tcp->rst)			/* Cannot reply to reset */
    return;

#ifdef SF_DEBUG_TCP
  printk("sf: tcp: send_tcp()\n");
#endif

  /* Swap ! */
  saddr = ip->daddr;
  daddr = ip->saddr;

  skb = dev_alloc_skb(len + dev->hard_header_len + 15);
  if (skb == NULL)
    return;
  /* IS_SKB(skb);   uncommented 19990505 hw*/

  /* skb->link3 = NULL; */
  skb->sk = NULL;
  /* skb->when = jiffies; */
  skb->pkt_type = PACKET_OTHERHOST;
  skb->csum = 0;

  /* skb->saddr = saddr; -- must be repaired  19990505 hw */
  /* skb->daddr = daddr; */

  /* skb->ip_hdr = (struct iphdr *) skb_put(skb, len); */
  skb->h.ipiph = (struct iphdr *) skb_put(skb, len);
  memcpy(skb->h.ipiph, ip, sizeof(struct iphdr));

  /* reverse packet, strip options */
  skb->h.ipiph->saddr = saddr;
  skb->h.ipiph->daddr = daddr;
  skb->h.ipiph->ttl = 255;
  skb->h.ipiph->tot_len = htons(len);
  skb->h.ipiph->frag_off = 0;
  skb->h.ipiph->ihl = 5;

  ip_route_output(&rt, daddr, saddr, 0, 0);  /* to be repaired... */
  /* rt = ip_rt_route(daddr, 0, NULL); */
  /* for Linux <= 2.0.30: rt = ip_rt_route(daddr, 0); */
  /* skb->localroute = 0; */
  /* skb->raddr = rt ? rt->rt_gateway : daddr; */

  /* Add the physical headers */
  ip_mksend(rt, skb, daddr, len, dev, saddr);

  ip_rt_put(rt);

  ip_send_check(skb->h.ipiph);

  /* change TCP header */
  tcp2 = (struct tcphdr *) (((char *) skb->h.ipiph) + (skb->h.ipiph->ihl << 2));
  memset(tcp2, 0, sizeof(struct tcphdr));

  tcp2->source = tcp->dest;
  tcp2->dest   = tcp->source;
  tcp2->doff   = sizeof(struct tcphdr) >> 2;

  /* RFC 793 */
  if (tcp->ack)
    tcp2->seq = tcp->ack_seq;
  else {
    tcp2->ack = 1;
    if (!tcp->syn)
      tcp2->ack_seq = tcp->seq;
    else
      tcp2->ack_seq = htonl(ntohl(tcp->seq) + 1);
  }
  if (rst)
    tcp2->rst = 1;

#ifdef SF_DEBUG_TCP
  printk("sf: tcp: %s seq=%08lx, ack_seq=%08lx\n", rst ? "RST" : "ACK",
         ntohl(tcp2->seq), ntohl(tcp2->ack_seq));
#endif

  /* Calculate TCP checksum */
  csum  = saddr & 0xffff;
  csum += saddr >> 16;
  csum += daddr & 0xffff;
  csum += daddr >> 16;
  csum += IPPROTO_TCP * 256;
  csum += htons(sizeof(struct tcphdr));

  sp = (__u16 *) tcp2;
  for (i = 0; i < sizeof(struct tcphdr) >> 1; i++, sp++)
    csum += *sp;
  while (csum >> 16)
    csum = (csum & 0xffff) + (csum >> 16);
  tcp2->check = ~csum;

  /* And send the packet... */
  dev_queue_xmit(skb /*, dev, 0 */ );

  dev_kfree_skb(skb);
}
